//
//  Scalar Ownage
//
//  Not really a game, but a tech demo of programming the Vectrex.
//  It shows how to use scaling, animation, collision detection.
// 
//  If you have any questions regarding the code, contact me
//  via the method specified at  http://www.sebastianmihai.com
//  (operational in 2012), and I will try my best to help.
//
//                             Sebastian Mihai, 2012
//


#include <vectrex.h>

/* NON CONSTANT VARIABLES NOT ALLOWED TO BE INITIALIZED */

/*
 * Some defines, Maximal brightness is $7f, highest not set!
 * Max Scale would be $ff, well here we take only $f0!
 */
#define MAX_BRIGHTNESS (0x7f)

#define SCALE 120                  /* for positioning the dots on the screen */
#define DOT_BRIGHTNESS 5                /* dot dwell time... */

#define ENEMY_SIZE 40
#define ANIMATION_FREQUENCY 30
#define ANIMATION_FRAMES 3
#define PLAYER_SPEED 2
#define BULLET_SPEED 10

static inline void initialize() {
	enable_joystick_1x();
	enable_joystick_1y();
	disable_joystick_2x();
	disable_joystick_2y();
	joy_digital();
	read_buttons();
	wait_recal();
	read_buttons();
	wait_recal();
}

static inline void put_dot() {
	write_ram(Vec_Dot_Dwell, DOT_BRIGHTNESS); /* first set up the dot dwell time */
	draw_dot( 0, 0 ); /* position and draw the dot */
}

static inline void line_to( char x, char y ) {
	write_ram(Vec_Dot_Dwell, DOT_BRIGHTNESS); /* first set up the dot dwell time */
	draw_line( x, y );
}

static inline void drawPlayer( signed char x ) {
	
	move_to(x, -120);
	move_to(0, -30 );
	
	move_to(-5, 0);
	
	line_to( 5, 15 );
	line_to( 5, -15 );
	line_to( -10, 0 );
	
	// back to middle and bottom of triangle
	move_to( 5, 0);
	
	// and back to origin
	move_to( -x, 120 );
	move_to(0, 30 );
}

static inline char firePressed() {
	if( joystick1_button1 || joystick1_button2 || joystick1_button3 || joystick1_button4 ) {
		return 1;
	}
	return 0;
}

static inline void drawBullet( signed char x, signed char y ) {
	move_to(x, y);
	line_to( 0, 5 );
	
	// back to origin
	move_to( 0, -5 );
	move_to( -x, -y );
}

static inline void drawEnemy( signed char x, signed char y, unsigned char frame ) {

	move_to(x, y);
	
	set_scale(frame);
	
	if( frame < ANIMATION_FREQUENCY ) {
		move_to( -ENEMY_SIZE/2, ENEMY_SIZE/2 );
		line_to( ENEMY_SIZE, 0 );
		line_to( 0, -ENEMY_SIZE );
		line_to( -ENEMY_SIZE, 0 );
		line_to( 0, ENEMY_SIZE );
		// back to center
		move_to( ENEMY_SIZE/2, -ENEMY_SIZE/2 );
	} else if ( frame < 2*ANIMATION_FREQUENCY ) {
		line_to( -ENEMY_SIZE/2, ENEMY_SIZE/2 );
		line_to( ENEMY_SIZE, 0 );
		line_to( -ENEMY_SIZE/2, -ENEMY_SIZE/2 );
		line_to( ENEMY_SIZE/2, -ENEMY_SIZE/2 );
		line_to( -ENEMY_SIZE, 0 );
		line_to( ENEMY_SIZE/2, ENEMY_SIZE/2 );
	} else {
		move_to( -ENEMY_SIZE/2, ENEMY_SIZE/2 );
		line_to( ENEMY_SIZE/2, -ENEMY_SIZE/2 );
		line_to( ENEMY_SIZE/2, ENEMY_SIZE/2 );
		line_to( 0, -ENEMY_SIZE );
		line_to( -ENEMY_SIZE/2, ENEMY_SIZE/2 );
		line_to( -ENEMY_SIZE/2, -ENEMY_SIZE/2 );
		line_to( 0, ENEMY_SIZE );
		// back to center
		move_to( ENEMY_SIZE/2, -ENEMY_SIZE/2 );
	}
	
	set_scale(SCALE);
	
	// back to origin
	move_to( -x, -y );
}

int game(void) {
	unsigned char i;
	signed char x; //player x
	signed char enemyX[5];
	signed char enemyY[5];
	signed char enemyAlive[5];
	signed char bulletX, bulletY;
	unsigned char frame=ANIMATION_FREQUENCY;
	signed char frameDelta = 2;
	unsigned char atLeastOneEnemyAlive;
	
	x = 0;
	initialize();
	bulletY = 127;
	bulletX = x;
	
	// enemy positions
	enemyX[0]=-100;enemyX[1]=-50;enemyX[2]=0;enemyX[3]=50;enemyX[4]=100;
	enemyY[0]=100;enemyY[1]=80;enemyY[2]=90;enemyY[3]=80;enemyY[4]=100;
	
	// As terrible as this repetition is, I could not get simple loops to work
	// It seems that during the later iterations in loops, the counter cannot be
	// used to index into arrays... couldn't figure out why.
	enemyAlive[0] = 1;
	enemyAlive[1] = 1;
	enemyAlive[2] = 1;
	enemyAlive[3] = 1;
	enemyAlive[4] = 1;
	
	while ( 1 )
	{
		frame+=frameDelta;
		
		//set_dp_c8();
		wait_recal();
		intensity(MAX_BRIGHTNESS);  /* set intensity of vector beam... */

		zero_beam();
		set_scale(SCALE);
		
		drawPlayer( x );
		if( bulletY <= 119 ) {
			drawBullet( bulletX, bulletY );
			bulletY+=BULLET_SPEED;
		} else {
			bulletX = -125;
		}

		// did bullet hit an enemy?
		for(i=0;i<5;i++) {
			if( bulletY + 5 > enemyY[i] &&
				bulletX > enemyX[i] - ENEMY_SIZE/3 &&
				bulletX < enemyX[i] + ENEMY_SIZE/3 ) {
				enemyAlive[i] = 0;
			}
		}

		// As terrible as this repetition is, I could not get simple loops to work
		// It seems that during the later iterations in loops, the counter cannot be
		// used to index into arrays... couldn't figure out why.
		if( enemyAlive[0] == 1 ) drawEnemy( enemyX[0], enemyY[0], frame );
		if( enemyAlive[1] == 1 ) drawEnemy( enemyX[1], enemyY[1], frame );
		if( enemyAlive[2] == 1 ) drawEnemy( enemyX[2], enemyY[2], frame );
		if( enemyAlive[3] == 1 ) drawEnemy( enemyX[3], enemyY[3], frame );
		if( enemyAlive[4] == 1 ) drawEnemy( enemyX[4], enemyY[4], frame );
		
		if(joystick1_x>0 && x < 127-PLAYER_SPEED) {
			x+=PLAYER_SPEED;
		} else if(joystick1_x<0 && x > -127 + PLAYER_SPEED) {
			x-=PLAYER_SPEED;
		}
		
		if( ( joystick1_y>0 || firePressed() ) && bulletY > 119 ) {
			// fire a new bullet!
			bulletY = -120;
			bulletX = x;
		}
		
		// reset frame to reset animation
		if( frame >= ANIMATION_FRAMES * ANIMATION_FREQUENCY ) {
			frameDelta = -2;
		} else if( frame <= ANIMATION_FREQUENCY ) {
			frameDelta = 2;
		}
		
		// bring beam to centre again, and print messages
		zero_beam();
		print_str(-127, 127,"SCALAR OWNAGE,SM 2012");
		
		joy_digital();
		read_buttons();

		// As terrible as this repetition is, I could not get simple loops to work
		// It seems that during the later iterations in loops, the counter cannot be
		// used to index into arrays... couldn't figure out why.
		if( enemyAlive[0] == 0 
		 && enemyAlive[1] == 0 
		 && enemyAlive[2] == 0
		 && enemyAlive[3] == 0
		 && enemyAlive[4] == 0)
			return;
	}
}

int showIntro(void) {
	initialize();
	while(1) {
		zero_beam();
		set_scale(SCALE);
		
		print_str(-67, 10,"PRESS FIRE");
		
		joy_digital();
		read_buttons();
		
		if( firePressed() ) {
			while( firePressed() ) {
				read_buttons();
			}
			return;
		}

	}
}

int main(void) {
	while(1) {
		showIntro();
		game();
	}
}